% GENERATE WAVE LOADS BASED ON RAO AND MEAN DRIFT COEFFICIENTS
%   Calculate linear wind sea wave forces based on JONSWAP-spectrum and 
%   linear transfer functions. Repeat procedure for swell sea.
%   Calculate slowly varying drift forces with Newman's approximation,
%   based on mean drift coefficients and JONSWAP-spectrum.
%   Add linear and nonlinear wave forces and create third input file for
%   USFOS-analysis.
tic
for SeaState = 1%:10
clearvars -except SeaState
% Define sea states


%
All_dir_WS = [0, 10, 20, 130, 140, 150, 170, 190, 210, 230];
% Multiply Hs with 1.09 to convert from 3 hour Hs to 1 hour Hs(DNV-OS-J101)
All_Hs_WS = [2.8, 2.8, 2.8, 2.5, 2.5, 2.4, 2.4, 2.4, 2.4, 2.4].*1.09;
All_Tp_WS = [6.6, 6.6, 6.6, 6.2, 6.2, 5.9, 5.9, 5.9, 5.9, 5.9];

All_dir_Swell = [160, 150, 140, 130, 160, 150, 140, 130, 160, 130];
% Multiply Hs with 1.09 to convert from 3 hour Hs to 1 hour Hs(DNV-OS-J101)
All_Hs_Swell = ...
    [0.34, 0.34, 0.34, 0.34, 0.34, 0.34, 0.34, 0.34, 0.34, 0.34].*1.09;
All_Tp_Swell = [12, 12, 12, 12, 18, 18, 18, 18, 20, 20];
%

% Define parameters for time domain analysis
Def_timePar

% Deine spectral parameters and parameters for wave condition
Def_waveCond
%%
% Generate spectrum for wind sea
[ S_WS ] = Gen_Jonswap( w, N, gam_WS, Hs_WS, w_p_WS, s_a, s_b );
% Generate spectrum for swell
[ S_Swell ] = Gen_Jonswap( w, N, gam_Swell, Hs_Swell, w_p_Swell, s_a, s_b);

%% PONTOON PROPERTIES

% Read x- and y-position of pontoons
Pont_file1 = fopen('x_positions.txt', 'r');
x_pos = fscanf(Pont_file1,'%f');
fclose(Pont_file1);

Pont_file2 = fopen('y_positions.txt', 'r');
y_pos = fscanf(Pont_file2,'%f');
fclose(Pont_file2);

% Read orientation of pontoons from file
Pont_file3 = fopen('Pontoon_angles.txt', 'r');
Pontoon_angles = fscanf(Pont_file3,'%f');
fclose(Pont_file3);

%Pontoon_angles = [Pontoon_angles; 0]; % Add angle of ship
% Round angles to closest 10
Pont_angRound = 10*round(Pontoon_angles/10);

% Vector giving the pontoon type for each pontoon (5 is the ship)
Pont_type = [1, 2*ones(1,3), 3*ones(1,6), 4*ones(1,36)];
%Pont_type = [1, 2*ones(1,3), 3*ones(1,6), 4*ones(1,36), 5];

% Create string for naming files
% Sea state should be uniquely defined by direction of wind sea and swell
% since Hs and Tp are defined for each heading angle.
% Heading angle given in coordinate system defined in report
FileName = sprintf('WS%i_Swell_%i_TpSwell_%i_LongCrested.fem', 100-dir_WS, 100-dir_Swell, Tp_Swell);


%% LINEAR WAVE FORCES

% Read linear transfer functions
[ TransferFunctions, PhaseAngles,N_RAO ] = ...
    Read_transferFunc(angles, '0.5', [1 2 3 4]);

% Read mean drift coefficients
[ MeanDriftCoefficients ] =Read_meanDriftCoeff_CFM(angles,'0.5',[1 2 3 4]);

% Interpolate transfer functions and phase angles
[ TransferFunctions ] = InterpolateCell( TransferFunctions, w, angles);
[ PhaseAngles ] = InterpolateCell( PhaseAngles, w, angles );
[ MeanDriftCoefficients ] = ...
                     InterpolateCell( MeanDriftCoefficients, w, angles);
%%
% WIND SEA
% Calculate deterministic amplitude for surface elevation
Ampl_WS = sqrt(2*S_WS*dw);

% Generate phase angles (all pontoons in phase if x=y=0), and store in file
%
epsi_WS = rand(1, N)*2*pi;
FID = fopen(strcat('Epsi_WS_',FileName), 'w');
fprintf(FID, '%1.4f\n', epsi_WS);
fclose(FID);
%

% SWELL SEA
% Calculate deterministic amplitude for surface elevation
Ampl_Swell = sqrt(2*S_Swell*dw);

% Generate phase angles (long crested), and store in file
%
epsi_Swell = rand(1, N)*2*pi;
FID = fopen(strcat('Epsi_Swell_',FileName), 'w');
fprintf(FID, '%1.4f\n', epsi_Swell);
fclose(FID);
%


%% GENERATE 3 HOUR FORCE HISTORIES
% Input file to USFOS: wave loads from full sea
inpf = fopen(strcat('Input_',FileName), 'w');

%inpf_static_WS = fopen(strcat('Static_WS_',FileName), 'w');
%inpf_static_Swell = fopen(strcat('Static_Swell_',FileName), 'w');

% Input file to USFOS: linear wave loads from wind sea
linf_WS = fopen(strcat('Input_linear_',FileName), 'w');

% Input file to USFOS: linear wave loads from swell
linf_Swell = fopen(strcat('Input_linear_Swell_',FileName), 'w');

% Input file to USFOS: nonlinear wave loads from wind sea
diffinf = fopen(strcat('Input_diff_inf_',FileName), 'w');


for P = 1: 46 % P = pontoon number, pontoons numbered from south to north
    p_type = Pont_type(P);% Pontoon type for locating right transfer func.
    
    % Short crested sea:
    %epsi_WS = rand(1, N)*2*pi;
    %epsi_Swell = rand(1, N)*2*pi;
    
    % Define angle in the same coordinate system as applied by Faltinsen
    % Wind sea
    Beta_WS = (dir_WS+90)*pi/180;
    %Swell
    Beta_Swell = (dir_Swell+90)*pi/180;
    
    % Create transfer matrix for the pontoon
    % Use exact angle
    [ TraMat ] = TransfMatrix( Pontoon_angles(P) );
    
    % WIND SEA
    if dir_WS >90
        dir_WS = dir_WS-180; % Force in the negative y-direction
    end
    rel_angle = Pont_angRound(P)+dir_WS; % Angle between wave and pontoon
    m = find(angles==abs(rel_angle)); % Index for the given angle
    
    % Calculate linear wind sea forces relative to local coordinate system
    [WS_forces]=Gen_linearForces(Ampl_WS,TransferFunctions{p_type}(:,:,m),...
   w, t, PhaseAngles{p_type}(:,:,m), epsi_WS, Beta_WS, x_pos(P), y_pos(P));
     
    % Write linear wind sea forces to file
     for a = 1:6 % DOF
     % Identify local maxima and minima and reduce size of file
     [ WS_red ] = Reduce_file( WS_forces(:,a), t', P );
     
     % Write results to correct file
     GenInpFile( linf_WS, TraMat, WS_red, P, a, t0);
     end     
     
    % SWELL SEA
    if dir_Swell >90
        dir_Swell = dir_Swell-180; % If in the negative y-direction
    end
    rel_angle = Pont_angRound(P)+dir_Swell; % Angle between wave and pontoon
    m_swell = find(angles==abs(rel_angle)); % Index for the given angle
    % Calculate linear swell sea forces relative to local coordinate system
    [Swell_forces]=Gen_linearForces(Ampl_Swell,...
          TransferFunctions{p_type}(:,:,m_swell), w, t,...
          PhaseAngles{p_type}(:,:,m_swell), epsi_Swell,...
          Beta_Swell, x_pos(P), y_pos(P));
     
    % Write swell forces to file
     for a = 1:6 % DOF
     % Identify local maxima and minima and reduce size of file
     [ Swell_red ] = Reduce_file( Swell_forces(:,a), t', P );
     
     % Write results to correct file
     GenInpFile( linf_Swell, TraMat, Swell_red, P, a, t0);
     end                       
                       
     % NONLINEAR WAVE FORCES
        % Only include wind sea
     
     % Difference frequency forces
     [ DiffFreqForces ] = Gen_nonLinearForces( Ampl_WS,...
         MeanDriftCoefficients{p_type}(:,:,m), w, t,...
         epsi_WS, Beta_WS, x_pos(P), y_pos(P), Pontoon_angles(P)*pi/180 );
     
     
     for a = 1:6
     % Filter out high frequency components
     [b_par,a_par]=butter(4,1/30,'low');
     DiffFreqForces_filtered(:,a)=filtfilt(b_par,a_par,DiffFreqForces(:,a));
     end
     
     
     % CALCULATE CHARACTERISTIC WAVE LOADS
     [ F_M_WS(P,:) ] = Find_charLoad( S_WS,dw, Tp_WS, ...
         TransferFunctions{p_type}(:,:,m), T_tot );
     [ F_M_Swell(P,:) ] = Find_charLoad( S_Swell,dw, Tp_Swell, ...
         TransferFunctions{p_type}(:,:,m), T_tot );
     
     % Write difference frequency forces to file
     for a = 1:6 % DOF
     % Identify local maxima and minima and reduce size of file
     [ DiffFreqForces_red ] = Reduce_file( DiffFreqForces_filtered(:,a), t', P );
     
     % Write results to correct file
     GenInpFile( diffinf, TraMat, DiffFreqForces_red, P, a, t0);
     end 
                            
     % Add forces
     Fexc_det = WS_forces+Swell_forces+ DiffFreqForces_filtered; 
         
     
     for a = 1:6 % DOF
     % Identify local maxima and minima and reduce size of file
     [ Fexc_red ] = Reduce_file( Fexc_det(:,a), t', P );
     
     % Write results to correct file
     GenInpFile( inpf, TraMat, Fexc_red, P, a, t0);
     end    
     
     %Gen_staticFile( inpf_static_WS, TraMat, P, F_M_WS(P,:), 'Wind Sea' );
     %Gen_staticFile( inpf_static_Swell, TraMat, P, F_M_Swell(P,:), 'Swell' );

end
fclose(inpf);
%fclose(inpf_static_WS);
%fclose(inpf_static_Swell);
fclose(linf_WS);
fclose(linf_Swell);
fclose(diffinf);
%{
% Write characteristic loads to file
F_M_WS = [F_M_WS; sum(F_M_WS)];
F_M_Swell = [F_M_Swell; sum(F_M_Swell)];

file_FM = fopen(strcat('Input_FM_WS_',FileName), 'w');
fprintf(file_FM, '%f   %f   %f   %f   %f   %f\n', F_M_WS');
fclose(file_FM);

file_FM = fopen(strcat('Input_FM_Swell_',FileName), 'w');
fprintf(file_FM, '%f   %f   %f   %f   %f   %f\n', F_M_Swell');
fclose(file_FM);
%}



end % End of loop with ULS condition SeaState
toc